#pragma once
#include "../pteidlib/pteidlib.h"
#include "PteidAddr.h"
#include "PteidCertif.h"
#include "PteidId.h"
#include "PteidPin.h"
#include "PteidPic.h"
#include "PteidRSAPublicKey.h"
#include "PteidTokenInfo.h"
#include "PteidProxyInfo.h"
#include "PteidDhParam.h"
#include "PteidDhAuthResponse.h"

using namespace System;
using namespace System::Runtime::InteropServices;

namespace eidpt 
{	
  	public enum class PteidCardType 
	{
		TYPE_ERR		= CARD_TYPE_ERR,		// Something went wrong, or unknown card type
		TYPE_IAS07		= CARD_TYPE_IAS07,   // IAS 0.7 card
		TYPE_IAS101		= CARD_TYPE_IAS101   // IAS 1.01 card
	};

  	public enum class PteidWebErrorCode{
		OK					= WEB_ERR_OK,			// OK: The previous operation was successful.
		ERR_SELECT_FILE		= WEB_ERR_SELECT_FILE,	// Select file error: There was an error finding a file in the card. Sw1 and sw2 are the response parameters.
		ERR_READ_FILE		= WEB_ERR_READ_FILE,	// Read Error: There was an error reading a file from the card. Sw1 and sw2 are the response parameters.
		ERR_WRITE_FILE		= WEB_ERR_WRITE_FILE,	// Write Error: There was an error writing on a file in the card. Sw1 and sw2 are the response parameters.
		ERR_BAD_COMMAND		= WEB_ERR_BAD_COMMAND,	// The card did not understand a generic command or refuses to serve it. Sw1 and sw2 are the response parameters.
		ERR_EMPTY_RES		= WEB_ERR_EMPTY_RES,	// Empty response: The data returned from the card is of zero size.
		ERR_DATA_SIZE		= WEB_ERR_DATA_SIZE,	// Data size not correct: The data returned from the card is not the correct size.
		ERR_CARD_REMOVED	= WEB_ERR_CARD_REMOVED,	// Card removed: The user removed the card during operation.
		ERR_CARD_COMM		= WEB_ERR_CARD_COMM,	// Card communication problem: The middleware can't communicate with the card.
		ERR_OUT_OF_MEM		= WEB_ERR_OUT_OF_MEM,	// Out of memory error.
		ERR_INTERNAL		= WEB_ERR_INTERNAL,		// Internal middleware error.
		ERR_PARSING			= WEB_ERR_PARSING,		// Error parsing server response: The content of the server response is invalid at the HTTP or JSON level.
		ERR_MISSING_DATA	= WEB_ERR_MISSING_DATA,	// Missing data in server response: The response from server lacks mandatory data.
		ERR_INVALID_HASH	= WEB_ERR_INVALID_HASH	// The card or the server has returned an invalid hash.
	};

	public ref class Pteid
    {
    public: 
        // Map result values
        ref class Result
        {
        public:
            static long OK					  = PTEID_OK				   ;
            static long E_BAD_PARAM	          = PTEID_E_BAD_PARAM	       ;
            static long E_INTERNAL            = PTEID_E_INTERNAL           ;
            static long E_INSUFFICIENT_BUFFER = PTEID_E_INSUFFICIENT_BUFFER;
            static long E_KEYPAD_CANCELLED	  = PTEID_E_KEYPAD_CANCELLED   ;
            static long E_KEYPAD_TIMEOUT	  = PTEID_E_KEYPAD_TIMEOUT     ;
            static long E_KEYPAD_PIN_MISMATCH = PTEID_E_KEYPAD_PIN_MISMATCH;
            static long E_KEYPAD_MSG_TOO_LONG = PTEID_E_KEYPAD_MSG_TOO_LONG;
            static long E_INVALID_PIN_LENGTH  = PTEID_E_INVALID_PIN_LENGTH ;
            static long E_NOT_INITIALIZED     = PTEID_E_NOT_INITIALIZED    ;
            static long E_UNKNOWN	          = PTEID_E_UNKNOWN	           ;
        };

        // See pteidlib.h voor comments
        static void                     Init(String^ readerName); 
        static void                     Exit(unsigned long mode); 
		static PteidCardType			GetCardType();
		static PteidId^                 GetID(); 
        static PteidAddr^               GetAddr();
        static PteidPic^                GetPic(); 
        static array<PteidCertif^>^     GetCertificates(); 
        static long                     VerifyPIN(Byte pinId, String^ pin); // Returns: number of tries left
        static long                     ChangePIN(Byte pinId, String^ pinOld, String^ pinNew); // Returns: number of tries left
        static array<PteidPin^>^        GetPINs();
        static PteidTokenInfo^          GetTokenInfo();
        static array<Byte>^             ReadSOD(); 
        static long                     UnblockPIN(Byte pinId, String^ puk, String^ pinNew); // Returns: number of tries left
        static long                     UnblockPIN_Ext(Byte pinId, String^ puk, String^ pinNew, unsigned long flags); // Returns: number of tries left
        static void                     SelectADF(array<Byte>^ adf);                               
        static array<Byte>^             ReadFile(array<Byte>^ file, Byte pinId);                   
        static void                     WriteFile(array<Byte>^ file, array<Byte>^ in, Byte pinId); 
        static unsigned long            IsActivated(); 
        static void                     Activate(String^ pin, array<Byte>^ pucDate, unsigned long mode);       
        static void                     SetSODChecking(int doCheck); 
        static void                     SetSODCAs(array<PteidCertif^>^ certificates);              
        static PteidRSAPublicKey^		GetCardAuthenticationKey(); 
        static PteidRSAPublicKey^		GetCVCRoot(); 
        static array<Byte>^             CVC_Init(array<Byte>^ pucCert); 
        static void                     CVC_Authenticate(array<Byte>^ pucSignedChallenge); 
        static array<Byte>^             CVC_ReadFile(array<Byte>^ file); 
        static void                     CVC_WriteFile(array<Byte>^ file, unsigned long fileOffset, array<Byte>^ fileIn, unsigned long mode);
        static PteidAddr^               CVC_GetAddr(); 
        static void                     CVC_WriteAddr(PteidAddr^ addrData); 
        static void                     CVC_WriteSOD(unsigned long fileOffset, array<Byte>^ fileIn, unsigned long mode); 

        static array<Byte>^             CVC_Init_SM101();
        static array<Byte>^             CVC_Authenticate_SM101( array<Byte>^ ifdChallenge,	
																array<Byte>^ ifdSerialNr,
																array<Byte>^ iccSerialNr,
																array<Byte>^ keyIfd,
																array<Byte>^ encKey,
																array<Byte>^  macKey);

        static PteidDhParam^            CVC_R_Init();
        static PteidDhAuthResponse^     CVC_R_DH_Auth(array<Byte>^ ucKifd, array<Byte>^ pucCert);
		static void						CVC_R_ValidateSignature(array<Byte>^ pucSignedChallenge);

		static array<Byte>^				SendAPDU(array<Byte>^ ucRequest);

		static void						ChangeAddress(String^ csServer,
														array<Byte>^ ucServerCaCert,
														PteidProxyInfo^ proxyInfo,
														String^ csSecretCode,
														String^ csProcess);

		static PteidAddressChangeState	GetChangeAddressProgress();
		//static void SetChangeAddressCallback(void(_USERENTRY * callback)(tAddressChangeState state));
		static void						CancelChangeAddress();

		static long						CAP_ChangeCapPin(String^ csServer, 
														array<Byte>^ ucServerCaCert,	
														PteidProxyInfo^ proxyInfo, 
														String^ pszOldPin,
														String^ pszNewPin);
		static PteidCapPinChangeState	CAP_GetCapPinChangeProgress();
		//static void						CAP_SetCapPinChangeCallback(void(_USERENTRY * callback)(tCapPinChangeState state));
		static void						CAP_CancelCapPinChange();

		static PteidWebErrorCode		GetLastWebErrorCode();
		static String^					GetLastWebErrorMessage();

   };

    public ref class PteidException : ApplicationException
    {        
    public:
        PteidException(long status): ApplicationException("PTEID Error Code : " + Convert::ToString(status))
        {
        }
    };
}

